function varargout = findFish(im_original,InitPoint,im_name,shapeModel,grayModel,varargin)

% Key-value pair varargin
keys = {'save_video','save_gif','visualize','facefinder','dist_metric','evolutions','vis_grid','layout'};
default_values = {0,0,1,'click','pca',4,0,'muct'};
[save_video, save_gif, vis, face_find_method, dist_metric, evolutions, vis_grid, layout] =...
    parseKeyValuePairs(varargin,keys,default_values);

% Save a video? A GIF?
if save_video || save_gif, close all, vis=1;
    if save_gif, videoFileName = [pwd filesep sprintf('ASM_FaceDetection_%s.gif',date)]; end
end
if save_video
    videoFileName = [pwd filesep sprintf('ASM_FaceDetection_%s',date)];
    global vidObj %#ok<*TLEV>
    vidObj = VideoWriter(videoFileName,'MPEG-4');
    vidObj.FrameRate = 50;
    open(vidObj); disp('Recording video...')
end

% Visualization stuff
% if vis_grid, vis=1; 
% end
tic

% Inverse of covariance matrix sometimes badly scaled
warning('off','MATLAB:nearlySingularMatrix');

% Shape model and constraints on new shapes
[~,V,D] = deal(shapeModel.meanShape,shapeModel.eVectors,shapeModel.eValues);

% How many resolution levels are in the gray model?
n_resolutions = length(grayModel);
search_sizes = round(linspace(8,3,n_resolutions));
if ~isscalar(evolutions)
    evolutions = round(linspace(evolutions(1),evolutions(2),n_resolutions));
else
    evolutions = repmat(evolutions,n_resolutions,1);
end

% Perform ASM search at each resolution level
for n_resolution = 1:n_resolutions
    GM = grayModel(n_resolution);
    
    % Smooth the image (Anti-aliasing?)
    h_filt = GM.Info.SmoothingFilter;
    im = conv2(h_filt,h_filt,im_original,'same');
    
    % Downsample everything that needs to be downsampled
    downsampleFactor = GM.Info.downsampleFactor;
    im = imresize(im,1/downsampleFactor);            % Resize image
    x_mean = shapeModel.meanShape./downsampleFactor; % Resize mean shape
    InitPoint = InitPoint./downsampleFactor;
    
    % Gray-level profiles model
    [g_mean, g_S, g_eVecs, g_eVals] = deal(GM.meanProfile, GM.covMatrix, GM.eVectors, GM.eValues);
    n_pcs = 5;
    if n_pcs >= size(g_eVecs{1},1)
        n_pcs = round(0.8*size(g_eVecs{1},1));
    end
    
    % Shape model constraints
    P = V(:,1:n_pcs);
    maxb=3*sqrt(D(1:n_pcs));
    
    % Sample a square region of pixels around each landmark
    rc = GM.Info.rc_squaresize; % Size of square (rc+1)
    n_landmarks = length(x_mean)/2;
    
    % Convolution mask (for gradient of the square regions)
    kernel = GM.Info.EdgeKernel; % Basically the image gradient
    C = GM.Info.SigmoidEQ; % Sigmoid equalization
    
    % Place mean shape over face (Can be automatic or user clicks on image)
    if n_resolution == 1
        x_original_estimate = placeShape(im,InitPoint,x_mean,'fish');
        
        %tic, drawnow
        [x_aligned,x_current] = deal(x_original_estimate);
        
        if save_video && ishandle(h_im)            
            for n = 1:5
                savevideo(h_im)
            end
        elseif save_gif && ishandle(h_im)
            savegif(h_im, videoFileName, 1)
            for n = 1:2 % Start with multiple frames at beginning of GIF
                savegif(h_im, videoFileName, 0)
            end
        end
    else
        resolutionScale = grayModel(n_resolution-1).Info.downsampleFactor/downsampleFactor;
        x_original_estimate = x_original_estimate*resolutionScale; % For final display
        x_current = x_current*resolutionScale;
        x_aligned = x_current; % Use the position determined from the lower resolution
        if vis, figure(h_im), hold off
            imshow(im,[]),plotLandmarks(x_aligned,'hold',1,'layout',layout), end
    end
   
    % Evolve estimate of face location, adjusting landmarks w/in model space
    if isscalar(evolutions), n_evolutions = evolutions;
    else
        n_evolutions = evolutions(n_resolution);
    end
    
    x_dist = zeros(n_landmarks, n_evolutions);
    for n_evolution = 1:n_evolutions
        if vis, title(sprintf('Downsample: 1/%d. %d remaining.\nEvolution %d/%d',...
                downsampleFactor,n_resolutions-n_resolution,n_evolution,n_evolutions),'fontsize',FS), drawnow('expose'), end
        x_suggested = zeros(size(x_aligned));
        
        for n_landmark = 1:n_landmarks
            %% Calculate 2D profiles near iPixel
            % Think of this as moving a square grid around iPixel and calculating a profile
            % for the grid in each shifted location
            search_size = search_sizes(n_resolution); % Shift amount in x,y directions (xy^2 locations in total)
            n_shift = 0;
            dist_min = [];
            iPixel_startingPosition = x_current(2*n_landmark+[-1 0]); % Starting position of current pixel for current evolution
            
            % Shift the 2D profile around the current landmark
            for c = -(search_size/2):(search_size/2)
                for r = -(search_size/2):(search_size/2)
                    n_shift = n_shift+1;
                    iPixel = iPixel_startingPosition+[r c]'; % Coordinates of current pixel
                    if vis_grid, plot(iPixel(1),iPixel(2),'bs'), hold on, end
                    
                    im_region = imcrop(im,[iPixel(1)-rc/2 iPixel(2)-rc/2 rc rc]);
                    sz = size(im_region);
                    if any(sz < rc+1)
                        im_region = padarray(im_region,max(rc-sz+1,0),'replicate','post');
                    end
                    if sz(1) > rc+1
                        im_region = im_region(1:rc,:);
                    end
                    if sz(2) > rc+1
                        im_region = im_region(:,1:rc);
                    end
                    
                    % Calculate the gradient for this region
                    k1 = [-1 0 1; -2 0 2; -1 0 1];
                    k2 = [-1 -2 -1; 0 0 0; 1 2 1];
                    im_region_filt1 = conv2(im_region,k1,'valid');
                    im_region_filt2 = conv2(im_region,k2,'valid');
                    im_region_filt = sqrt(im_region_filt1.^2+im_region_filt2.^2);
                    im_region_filt = conv2(im_region,kernel,'valid');
%                     abs_sum = sum(abs(im_region_filt(:)));
%                     if abs_sum ~= 0
%                         im_region_filt = im_region_filt./abs_sum;
%                     end
                    im_region_filt = im_region_filt - min(im_region_filt(:));
                    im_region_filt = im_region_filt ./ max(im_region_filt(:));
                    
                    % Sigmoid equalization
                    im_region_filt = im_region_filt./(abs(im_region_filt)+C);
                    
                    % Store the current 2D profile as a 1D vector
                    g_new = reshape(im_region_filt,size(im_region_filt,1).^2,1);
                    
                    % Compute the 'distance' from the current profile to the mean profile
                    g_bar = g_mean{n_landmark};
%                     tt = size(im_region_filt,1);
%                     BB = zeros(tt, tt);
%                     for n1=1:tt
%                         BB(:,n1) = g_bar((n1-1)*tt+1:(n1-1)*tt + tt);
%                     end
                    if strcmpi(dist_metric,'pca')
                        % Approximate current gray profile in the gray model space
                        g_P = g_eVecs{n_landmark}(:,1:n_pcs); eVals = g_eVals{n_landmark}(1:n_pcs);
                        g_new_b = g_P'*(g_new-g_bar); % weights for suggested shape
                        
                        % How well does the model approximation fit the mean profile?
                        R = (g_new-g_bar)'*(g_new-g_bar) - g_new_b'*g_new_b; % This is actually R^2
                        F = sum((g_new_b.^2)./eVals) + 2*(R./eVals(end)); % Not sure if the second term should be in the sum
                        dist = F;
                    elseif strcmpi(dist_metric,'maha')
                        % Measure Mahalanobis distance for this shift
                        % (This is the distance between this image's profile at the current
                        % shift, compared to the mean profile for this landmark from the
                        % gray-level model that has been passed into this function)
                        md = (g_new-g_bar)'*inv(g_S{n_landmark})*(g_new-g_bar);
                        
                        % Should I be doing the abs(md) ?
                        md = abs(md);
                        dist = md;
                    end
                    
                    % Keep track of best shift
                    if isempty(dist_min)
                        dist_min = dist;
                        best_pixel = iPixel';
                    elseif dist < dist_min
                        dist_min = dist;
                        best_pixel = iPixel';
                    end
                end
            end % End shifting square grid around iPixel
            
            % TODO: Move the distance calculations outside of the loops
            x_suggested(2.*n_landmark+[-1 0]) = best_pixel;
            x_dist(n_landmark, n_evolution) = dist_min;
            
            % Visualize & save video (optional)
            if vis_grid, plot(best_pixel(1),best_pixel(2),'y.'), drawnow(), end
            if save_video && ishandle(h_im)
                savevideo(h_im)
            end
        end % Looping through landmarks
%         for n_landmark = 1:n_landmarks
%             if x_dist(n_landmark, n_evolution) > 10
%                 x_suggested(2.*n_landmark-1) = x_current(2.*n_landmark-1);
%                 x_suggested(2.*n_landmark) = x_current(2.*n_landmark);
%             end
%         end
        
        W = x_dist(:, n_evolution);
        W = 1-W/max(W);
        W = W/sum(W);
        % Update pose parameters towards suggested shape
        [~,x_posed] = procrustes(x_suggested,x_current);
        %x_posed = procrust(x_suggested, W, x_current);
        
%         old_x_posed = x_posed;
%         [~,x_pn] = procrustes([x_mean(1:2:end), x_mean(2:2:end)], [x_posed(1:2:end), x_posed(2:2:end)]);
%         x_posed(1:2:end) = x_pn(:,1);
%         x_posed(2:2:end) = x_pn(:,2);

        % Deform shape towards suggested points
        b_suggested = P'*(x_posed-x_mean);
        b=max(min(b_suggested,maxb),-maxb); % Keep adjustments within model limits
        
        % Generate new shape (within model space)
        x_new = x_mean + P*b;
        
        % Transfer x_new to image space
        xn = [x_new(1:2:end), x_new(2:2:end)]; xs = [x_suggested(1:2:end), x_suggested(2:2:end)];
        [~, xn] = procrustes(xs, xn);
        x_new = zeros(size(x_mean));
        x_new(1:2:end) = xn(:,1); x_new(2:2:end) = xn(:,2);
        x_prev = x_current;
        x_current = x_new;
        
        if vis % View the current evolution
            imshow(im,[]), plotLandmarks(x_current,'hold',1,'linewidth',3,'layout',layout), hold on
            plot(x_suggested(1:2:end),x_suggested(2:2:end),'bo','linewidth',4), drawnow()
            plot(x_posed(1:2:end),x_posed(2:2:end),'go','linewidth',4), drawnow()
            plot(x_prev(1:2:end),x_prev(2:2:end),'yo','linewidth',4), drawnow()
            
            if save_video && ishandle(h_im)
                savevideo(h_im)
            elseif save_gif && ishandle(h_im)
                savegif(h_im, videoFileName)
            end
        end
    end % End looping through evolutions
end % End looping through resolution levels

% Scale the final shape estimation to original image size
x_final = x_new*downsampleFactor;
x_original_estimate = x_original_estimate*downsampleFactor; % For final display

toc

x_dist = mean(x_dist);
%disp(mean(x_dist'));
if mean(x_dist') < 10
    a=dir([sprintf('../../Data/images_segmented/%s/', im_name) 'binary*']);
    it=size(a,1);
    normI = im_original - min(im_original(:));
	normI = normI ./ max(normI(:));
    set(figure, 'Visible', 'off'), imshow(imadjust(normI),[]);
    hold on,plot(x_original_estimate(1:2:end,1),x_original_estimate(2:2:end,1),'ro','MarkerSize', 1), hold off
    hold on,plot([x_final(1:2:end,1);x_final(1)],[x_final(2:2:end,1);x_final(2)],'-go','MarkerSize', 1), hold off
    saveas(gcf, sprintf('../../Data/images_segmented/%s/segment_%02d.png', im_name, it+1));

    A = zeros(512, 960);

    point1 = [x_final(2); x_final(1)];
    for l = 1:n_landmarks-1
        point2 = [x_final(2*(l+1)); x_final(2*(l+1)-1)];

        A = insertShape(A,'Line',[point1(2) point1(1) point2(2) point2(1)],'LineWidth',1,'Color','white','SmoothEdges', false);
        point1 = point2;
    end
    point2 = [x_final(2); x_final(1)];
    A = insertShape(A,'Line',[point1(2) point1(1) point2(2) point2(1)],'LineWidth',1,'Color','white','SmoothEdges', false);
    A = A(:,:,1); % make binary again

    % region growing using 4-connectivity to fill shape 
    seed = [round(sum(x_final(1:2:end))/n_landmarks), round(sum(x_final(2:2:end))/n_landmarks)];
    %A(seed(2), seed(1)) = 1;
    Control = zeros(512, 960);
    s = seed(1, :)';
    Control(s(2), s(1)) = 1;

    while size(seed) ~= 0
        s = seed(1, :)';
        A(s(2), s(1)) = 1;
        if s(2)==512 || s(2)==1
            break;
        end
        if s(1)==960 || s(1)==1
            break;
        end

        if A(s(2), s(1)+1) == 0
            if Control(s(2), s(1)+1) == 0
                seed = [seed ; [s(1)+1,s(2)]];
            end
        end
        if A(s(2)+1, s(1)) == 0
            if Control(s(2)+1, s(1)) == 0
                seed = [seed ; [s(1),s(2)+1]];
            end
        end
        if A(s(2), s(1)-1) == 0
            if Control(s(2), s(1)-1) == 0
                seed = [seed ; [s(1)-1,s(2)]];
            end
        end
        if A(s(2)-1, s(1)) == 0
            if Control(s(2)-1, s(1)) == 0
                seed = [seed ; [s(1),s(2)-1]];
            end
        end

        Control(s(2), s(1)+1) = 1;
        Control(s(2)+1, s(1)) = 1;
        Control(s(2), s(1)-1) = 1;
        Control(s(2)-1, s(1)) = 1;
        seed = seed(2:end, :);
    end

    phead = [x_final(30*2-1); x_final(30*2)];
    ptail = [x_final(end-1); x_final(end)];

    ptop = [x_final(2*11-1); x_final(2*11)];
    pbot = [x_final(2*(60-11)-1); x_final(2*(60-11))];

    fishlength = round(norm(phead - ptail));
    fishheight = round(norm(ptop - pbot));

    imwrite(A, sprintf('../../Data/images_segmented/%s/binary_%02d_%d_%d.png', im_name, it+1, fishlength, fishheight));
end

% xx = x_final(1:2:end);
% yy = x_final(2:2:end);
% fileID = fopen('fish.csv','a');
% fprintf(fileID, '%s,0000', im_name);
% 
% for j=1:size(xx, 1)
%     fprintf(fileID, ',%d,%d', round((xx(j))-960/2), round(((yy(j))-512/2)*-1));
% end
% fprintf(fileID, '\r\n');
% 
% fclose(fileID);

% Compare the original estimate with the final evolution
if vis
    figure(gcf), hold off, imshow(im_original,[])
    %imGO = image(im_original,'CDataMapping','scaled');
    %colormap('gray');
    %set(gcf, 'units','normalized','outerposition',[0 0 1 1]);
    hold on,plot(x_original_estimate(1:2:end,1),x_original_estimate(2:2:end,1),'ro','MarkerSize', 1), hold off
    %h_orig  = plotLandmarks(x_original_estimate,'hold',1,'linestyle','--','layout',layout);
    hold on,plot([x_final(1:2:end,1);x_final(1)],[x_final(2:2:end,1);x_final(2)],'-go','MarkerSize', 1), hold off
    %h_final = plotLandmarks(x_final,'hold',1,'color','g','linewidth',2,'layout',layout);
    %legend([h_orig,h_final],{'Original','Final'},'location','nw','fontsize',FS);
    %title('Final shape','fontsize',FS);
    
    if save_video        
        for n = 1:30 % Add extra copies of the final image
            savevideo(h_im)
        end
        close(vidObj), disp('Video closed.') % Close video                
    elseif save_gif && ishandle(h_im)
        for n = 1:6
            savegif(h_im, videoFileName)
        end
    end
end

% Output final shape and model parameters
if nargout == 1
    varargout{1} = x_final;
elseif nargout == 2
    varargout{1} = x_final;
    varargout{2} = b;
end

end % End of main

%% --------------------------------------------------------------------
% Internal functions
%  --------------------------------------------------------------------

function savevideo(im_handle)
% SAVEVIDEO saves a single frame to the global video object.
    global vidObj
    
    drawnow()
    try frame = getframe(im_handle);
        writeVideo(vidObj,frame);
    catch
        close(vidObj)
    end
end

function savegif(im_handle, filename, first_frame)
% SAVEGIF saves a single frame to the GIF file.
    if nargin < 3
        first_frame = false;
    end
    
    drawnow()
    gif_im = frame2im(getframe(im_handle));
    [imind,cm] = rgb2ind(gif_im,256);
    if first_frame
        imwrite(imind,cm,filename,'gif','LoopCount',inf);
    else
        imwrite(imind,cm,filename,'gif','WriteMode','append','delaytime',0.5);
    end
end
